unit BDEDoRx4;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, DB, DBTables, DBIProcs, DBITypes;

type
  TIntIndexOpt = (iixUnknown, iixSingle, iixComp);

  TAddIdxFrm = class(TForm)
    Edit1: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Memo1: TMemo;
    PrimChBx: TCheckBox;
    UniChBx: TCheckBox;
    DescChBx: TCheckBox;
    CaseInsChBx: TCheckBox;
    ExprChBx: TCheckBox;
    DoItBtn: TButton;
    CancelBtn: TButton;
    Bevel2: TBevel;
    SelFieldsButton: TButton;
    SmartModeChBx: TCheckBox;
    Bevel1: TBevel;
    Memo2: TMemo;
    SelDescFieldsBtn: TButton;
    procedure FormCreate(Sender: TObject);
    procedure DoItBtnClick(Sender: TObject);
    procedure SelFieldsButtonClick(Sender: TObject);
    procedure SetControlStatus(Sender: TObject);
    procedure EditsChange(Sender: TObject);
    procedure SmartModeChBxClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private-Deklarationen }
    FIsSmartMode: boolean;
    FIsPdx: boolean;
    FIntIndexOpt: TIntIndexOpt;
    FCalced: boolean;
  public
    { Public-Deklarationen }
    FIndexName: string;
  end;

var
  AddIdxFrm: TAddIdxFrm;

implementation

uses BDEDoRx1, BDEDoRx5, BDEDoRxS;

{$R *.DFM}
{$B-;XL+}

const Label2Caps: array[Boolean] of string = ('Key Fields:',
                                                'Expression:');

procedure TAddIdxFrm.FormCreate(Sender: TObject);
begin
  FIntIndexOpt := iixUnknown;
  FIsPdx := (StrPas(MainFrm.FCurProps.szTableType) = szParadox);

  CalcControlSize(self);
  SmartModeChBxClick(Sender);

  if FIsPdx and (MainFrm.FCurProps.iTbllevel >= 7) then
  begin
    Memo2.Visible := True;
    with SelDescFieldsBtn do
    begin
      Visible := True;
      SetBounds((Memo2.Left+Memo2.Width)-Width,
              (Memo2.Top+Memo2.Height)-Height, Width, Height);
    end;
  end
  else
  begin
    Memo2.Visible := False;
    SelDescFieldsBtn.Visible := False;
    Memo1.Height := (Memo1.Height+Memo2.Height+8);
  end;
  with SelFieldsButton do
    SetBounds((Memo1.Left+Memo1.Width)-Width,
              (Memo1.Top+Memo1.Height)-Height, Width, Height);
  SetControlStatus(self);
end;

procedure RemoveChars(var S: string; Chars: array of Char);
var i: integer;
begin
  for i:=0 to high(Chars) do
    while (pos(Chars[i],S)<>0) do
      delete(S, pos(Chars[i],S), 1);
end;

procedure TAddIdxFrm.DoItBtnClick(Sender: TObject);
var IdxName, KeyExp, DescExp: string;
    IdxOptions: TIndexOptions;
    i,j: integer;
begin {DoItButtonClick}
  KeyExp := '';
  DescExp := '';
  IdxName := '';
  IdxOptions := [];
  Screen.Cursor := crHourGlass;
  try
    for i := 0 to pred(ComponentCount) do
    begin
      if (Components[i] is TCheckBox)
      and (TCheckBox(Components[i]).Checked) then
        case TCheckBox(Components[i]).Tag of
          1: Include(IdxOptions, ixPrimary);
          2: Include(IdxOptions, ixUnique);
          3: Include(IdxOptions, ixDescending);
          4: Include(IdxOptions, ixCaseInsensitive);
          5: Include(IdxOptions, ixExpression);
        end;
    end;

    KeyExp := Memo1.Text;
    RemoveChars(KeyExp, [#10,#13]);

    DescExp := Memo2.Text;
    RemoveChars(DescExp, [#10,#13]);

    IdxName := Edit1.Text;

    with MainFrm.Table1 do
    begin
      if (self.Tag = 1) then
      begin
        if (ixPrimary in IdxOptions) then
        begin
          BDEAlterPrimary(MainFrm.Table1, KeyExp, IdxOptions);
          (self).ModalResult := mrOk;
          Exit;
        end
        else
          MainFrm.Table1.DeleteIndex(FIndexName)
      end;
      if FIsPdx and (ixDescending in IdxOptions) then
        AddPx7Index(IdxName, KeyExp, DescExp, IdxOptions)
      else {}
        AddIndex(IdxName, KeyExp, IdxOptions);
    end;

  finally
    Screen.Cursor := crDefault;
  end;
  (self).ModalResult := mrOk;
end;

procedure TAddIdxFrm.SelFieldsButtonClick(Sender: TObject);
const SelFrmCaption: array[boolean] of string =
      ('Select Index Fields', 'Select Descending Fields');
var SKeyFields: string;
    SMemo: TMemo;
    i: integer;
begin
  SKeyFields := '';
  SelFldsFrm := TSelFldsFrm.Create(Application);
  with SelFldsFrm do
  begin
    try try
      if (Sender = SelDescFieldsBtn) then
        SMemo := Memo2
      else
        SMemo := Memo1;
      FIndexFields := SMemo.Text;
      Top := (self).Top+10;
      if (self).Left+(Self).Width+(Width div 2) < Screen.Width then
        Left := (self).Left+(Self).Width-(Width div 2)
      else Left := (self).Left-Width+(Width div 2);

      Caption := SelFrmCaption[(Sender = SelDescFieldsBtn)];
      SetMoveMode(False);
      if (FIsPdx and PrimChBx.Checked) then
        SetPrimary;
      MainFrm.Table1.GetFieldNames(ListBox1.Items);
      ShowModal;
    except
      Free;
      SelFldsFrm := nil;
      raise;
    end;

    with ListBox2 do
      if (ModalResult = mrOK)
      and (Items.Count > 0) then
      begin
        for i:=0 to pred(Items.Count) do
          SKeyFields := SKeyFields+Items[i]+';';
        {$IFDEF WIN32}
        SetLength(SKeyFields,pred(Length(SKeyFields)));
        {$ELSE}
        SKeyFields[0] := pred(SKeyFields[0]); {}
        {$ENDIF}
        SMemo.Lines.Clear;
        SMemo.Lines.Add(SKeyFields);
        if (Sender = SelFieldsButton) then
        begin
          Edit1.Enabled := True;
          FIntIndexOpt := TIntIndexOpt(ord(Items.Count > 1)+1);
          if FIsPdx and DescChBx.Checked then
          begin
            {Memo2.Enabled := True;}
            Memo2.Lines.Clear;
            Memo2.Lines.Add(SKeyFields);
          end;
        end;
      end;
    finally
      SelFldsFrm.Free;
      SelFldsFrm := nil;
    end;
  end;
  SetControlStatus(Sender);
end;

procedure TAddIdxFrm.EditsChange(Sender: TObject);
begin
  DoItBtn.Enabled := ((length(Memo1.Text)>0)
                       and ((length(Edit1.Text)>0)
                            or PrimChBx.Checked));
end;

procedure TAddIdxFrm.SetControlStatus(Sender: TObject);
begin
  if not FIsSmartMode then exit;
  PrimChBx.Enabled := FIsPdx and ((MainFrm.FCurProps.iIndexes = 0)
                      or ((Tag = 1) and (FIndexName = '')));
  PrimChBx.Checked := PrimChBx.Enabled;

  UniChBx.Enabled := (not FIsPdx)
                      or ((MainFrm.FCurProps.iTblLevel >= 7)
                      and not (MainFrm.FCurProps.iIndexes = 0));
  DescChBx.Enabled := (not FIsPdx)
                      or ((MainFrm.FCurProps.iTblLevel >= 7)
                      and not (MainFrm.FCurProps.iIndexes = 0));
  CaseInsChBx.Enabled := FIsPdx and (MainFrm.FCurProps.iTblLevel >= 4)
                       and ((MainFrm.FCurProps.iIndexes <> 0)
                       and not ((Tag = 1) and (FIndexName = '')));
  ExprChBx.Enabled := not FIsPdx;
  if (not FIsPdx) and (FIntIndexOpt = iixComp) then
    ExprChBx.Checked := True;

  with Edit1 do
  begin
    if Enabled
    and FIsPdx
    and not CaseInsChBx.Checked
    and (FIntIndexOpt = iixSingle)
    {and (SelFldsFrm <> nil) {}
    and not PrimChBx.Checked
    and not DescChBx.Checked
    and not UniChBx.Checked then
      Text := Memo1.Lines[0];
    if Enabled and PrimChBx.Checked then
      Text := '';
    Enabled := not (PrimChBx.Checked
                    or (FIsPdx
                        and (not CaseInsChBx.Checked)
                        and (FIntIndexOpt = iixSingle)))
               or (FIsPdx and (DescChBx.Checked
                               or UniChbx.Checked));
  end;
  {Memo1.Enabled := ExprChBx.Checked;}
  Memo1.ReadOnly := not ExprChBx.Checked;
  SelDescFieldsBtn.Enabled := FIsPdx and DescChBx.Checked;
  if FIsPdx and DescChBx.Enabled then
  begin
    {Memo2.Enabled := True;}
    if DescChBx.Checked
    and (Memo2.Text = '')
    and (Memo1.Text <> '') then
      Memo2.Lines.Assign(Memo1.Lines)
    else
      if (not DescChBx.Checked)
      and (Memo2.Text <> '') then
        Memo2.Lines.Clear;
  end;
  {Memo2.Enabled := False;}
  Memo2.ReadOnly := True;

  Label2.Caption := Label2Caps[ExprChBx.Checked];
  DoItBtn.Enabled := (length(Memo1.Text)>0)
                       and ((length(Edit1.Text)>0)
                           or PrimChBx.Checked);
end;

procedure TAddIdxFrm.SmartModeChBxClick(Sender: TObject);
var i: integer;
begin
  FIsSmartMode := SmartModeChBx.Checked;
  if not FIsSmartMode then
    for i:=0 to pred(ComponentCount) do
      if (Components[i] is TMemo) then
        TMemo(Components[i]).ReadOnly := False
      else
        if (Components[i] is TWinControl) then
          TWinControl(Components[i]).Enabled := True;
  SetControlStatus(Sender);
  EditsChange(Sender);
end;

procedure TAddIdxFrm.FormShow(Sender: TObject);
var i: integer;
begin
  if not FCalced and (Tag = 1) then {alter index}
  begin
    with MainFrm.Table1.IndexDefs do
    begin
      Update;
      for i:=0 to pred(Count) do
        if (Items[i].Name = FIndexName) then
        begin
          Edit1.Text := FIndexName;
          Memo1.Text := Items[i].Fields;
          if not (ixPrimary in Items[i].Options)
          and (Pos(';',Items[i].Fields) > 0) then
            FIntIndexOpt := iixComp
          else
            FIntIndexOpt := iixSingle;
          if Memo2.Visible then
            Memo2.Text := MainFrm.Table1.GetDescFields(FIndexName);
          PrimChBx.Checked := (ixPrimary in Items[i].Options);
          UniChBx.Checked := (ixUnique in Items[i].Options)
                             and not (ixPrimary in Items[i].Options);
          DescChBx.Checked := (ixDescending in Items[i].Options);
          CaseInsChBx.Checked := (ixCaseInSensitive in Items[i].Options);
          ExprChBx.Checked := (ixExpression in Items[i].Options);
        end;
    end;
    SmartModeChBxClick(Sender);
    FCalced := True;
  end;
end;

initialization
begin
  AddIdxFrm := nil;
end;

end.



